"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Content_contentElement;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Content = void 0;
/**
 * List of invalid attributes we should remove part of our
 * sanitizer.
 */
const invalidAttributes = ["lang", "dir", "accessKey", "tabIndex"];
/**
 * This list includes attributes we want to further operate
 * on but which should otherwise be left alone.
 */
const validAttributes = ["style", "src"];
/**
 * HTML output from Evernote is relatively clean but there
 * are a lot of domain-specific inline styles. This list serves
 * as a very basic inline styles sanitizer.
 */
const validStyles = [
    "background-color",
    "color",
    "text-align",
    "font-family",
    "font-size",
];
/**
 * This is a list of special elements used by Evernote for different
 * purposes.
 */
const invalidElements = [
    "en-media",
    "en-crypt",
    "en-todo",
    // The new editor by evernote includes new special elements.
    // However, these elements are not mentioned in the spec nor
    // do they appear in the exported files (yet). I am adding them
    // here just in case.
    "en-codeblock",
    "en-task-group",
];
const cssSelector = [
    ...validAttributes.map((attr) => `[${attr}]`),
    ...invalidAttributes.map((attr) => `[${attr}]`),
    ...invalidElements,
].join(",");
class Content {
    constructor(contentElement) {
        _Content_contentElement.set(this, void 0);
        __classPrivateFieldSet(this, _Content_contentElement, contentElement, "f");
    }
    toHtml(handler) {
        return __awaiter(this, void 0, void 0, function* () {
            const noteElement = __classPrivateFieldGet(this, _Content_contentElement, "f").querySelector("en-note");
            if (!noteElement)
                throw new Error("Invalid content.");
            const elements = noteElement.querySelectorAll(cssSelector);
            for (let element of elements) {
                let elementType = filterAttributes(element) || element.tagName.toLowerCase();
                switch (elementType) {
                    case "img-dataurl":
                    case "en-codeblock":
                    case "en-task-group":
                    case "en-crypt":
                    case "en-todo":
                    case "en-media": {
                        if (handler) {
                            const result = yield handler.process(elementType, element);
                            if (!!result)
                                element.replaceWith(result);
                            else
                                element.remove();
                        }
                        else
                            element.remove();
                        break;
                    }
                }
            }
            return noteElement.innerHTML;
        });
    }
    get raw() {
        const noteElement = __classPrivateFieldGet(this, _Content_contentElement, "f").querySelector("en-note");
        if (!noteElement)
            throw new Error("Invalid content.");
        return noteElement.innerHTML;
    }
}
exports.Content = Content;
_Content_contentElement = new WeakMap();
function stylesToObject(input) {
    const styles = input.split(";");
    const output = {};
    for (let style of styles) {
        const [key, value] = style.trim().split(":");
        output[key] = value;
    }
    return output;
}
function objectToStyles(input) {
    const output = [];
    for (let key in input) {
        output.push(`${key}:${input[key]}`);
    }
    return output.join(";");
}
function filterAttributes(element) {
    let elementType = null;
    for (let attr of invalidAttributes) {
        if (element.hasAttribute(attr))
            element.removeAttribute(attr);
    }
    for (let attr of validAttributes) {
        if (!element.hasAttribute(attr))
            continue;
        const value = element.getAttribute(attr);
        if (!value) {
            element.removeAttribute(attr);
            continue;
        }
        switch (attr) {
            case "src": {
                if (!value.startsWith("data:image/"))
                    break;
                elementType = "img-dataurl";
                break;
            }
            case "style": {
                const styles = stylesToObject(value);
                for (let style in styles) {
                    switch (style) {
                        case "--en-codeblock":
                            elementType = "en-codeblock";
                            break;
                        case "--en-task-group":
                            elementType = "en-task-group";
                            const taskGroupId = styles["--en-id"];
                            if (taskGroupId)
                                element.setAttribute("task-group-id", taskGroupId);
                            break;
                    }
                    if (validStyles.indexOf(style) === -1)
                        delete styles[style];
                }
                const newStyle = objectToStyles(styles);
                if (newStyle)
                    element.setAttribute(attr, newStyle);
                else
                    element.removeAttribute(attr);
                break;
            }
        }
    }
    return elementType;
}
