"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAsDateRequired = exports.getAsDate = exports.getAsBoolean = exports.getAsNumberRequired = exports.getAsNumber = exports.getAsStringRequired = exports.getAsString = void 0;
const iso8601datetime_1 = require("./iso8601datetime");
const node_html_parser_1 = require("node-html-parser");
function getAsString(element, tagName) {
    const tag = findChild(element, tagName);
    if (!tag)
        return null;
    return tag.textContent.trim();
}
exports.getAsString = getAsString;
function getAsStringRequired(element, tagName) {
    const tag = findChild(element, tagName);
    if (!tag || !tag.textContent)
        throw new Error(`${tagName} is required.`);
    return tag.textContent.trim();
}
exports.getAsStringRequired = getAsStringRequired;
function getAsNumber(element, tagName) {
    const tag = parseFloat(getAsString(element, tagName) || "");
    if (isNaN(tag))
        return null;
    return tag;
}
exports.getAsNumber = getAsNumber;
function getAsNumberRequired(element, tagName) {
    const tag = parseFloat(getAsStringRequired(element, tagName));
    if (isNaN(tag))
        throw new Error(`${tagName} value is an invalid number.`);
    return tag;
}
exports.getAsNumberRequired = getAsNumberRequired;
function getAsBoolean(element, tagName) {
    const tag = getAsString(element, tagName);
    return tag === "true";
}
exports.getAsBoolean = getAsBoolean;
function getAsDate(element, tagName) {
    const time = getAsString(element, tagName);
    if (!time)
        return null;
    return iso8601datetime_1.ISO8601DateTime.toDate(time);
}
exports.getAsDate = getAsDate;
function getAsDateRequired(element, tagName) {
    const time = getAsStringRequired(element, tagName);
    const date = iso8601datetime_1.ISO8601DateTime.toDate(time);
    if (!date)
        throw new Error(`${tagName} value is an invalid date.`);
    return date;
}
exports.getAsDateRequired = getAsDateRequired;
function findChild(element, tagName) {
    for (let child of element.childNodes) {
        if (child.nodeType === node_html_parser_1.NodeType.ELEMENT_NODE &&
            child.tagName.toLowerCase() === tagName.toLowerCase()) {
            return child;
        }
    }
    return null;
}
